@@ -227,7 +227,7 @@ fn run(matches: &clap::ArgMatches) -> Result<()> {
     let stack = Stack::current(&repo, InitializationPolicy::AutoInitialize)?;
     let stupid = repo.stupid();
 
-    let source_path = if matches.get_flag("url") {
+    let source_path = if cfg!(feature = "import-url") && matches.get_flag("url") {
         None
     } else if let Some(path) = matches.get_one::<PathBuf>("source") {
         let abs_path = gix::path::realpath(path)?;
@@ -312,7 +312,6 @@ fn import_url(stack: Stack, matches: &clap::ArgMatches) -> Result<()> {
     }
 }
 
-#[cfg(feature = "import-compressed")]
 fn import_tgz_series(stack: Stack, matches: &clap::ArgMatches, source_path: &Path) -> Result<()> {
     let source_file = std::fs::File::open(source_path)?;
     let mut archive = tar::Archive::new(flate2::read::GzDecoder::new(source_file));
@@ -322,17 +321,15 @@ fn import_tgz_series(stack: Stack, matches: &clap::ArgMatches, source_path: &Pat
     return import_series(stack, matches, Some(series_path.as_path()));
 }
 
-#[cfg(feature = "import-compressed")]
 fn import_tbz2_series(stack: Stack, matches: &clap::ArgMatches, source_path: &Path) -> Result<()> {
     let source_file = std::fs::File::open(source_path)?;
-    let mut archive = tar::Archive::new(bzip2::read::BzDecoder::new(source_file));
+    let mut archive = tar::Archive::new(bzip2_rs::DecoderReader::new(source_file));
     let temp_dir = tempfile::tempdir()?;
     archive.unpack(temp_dir.path())?;
     let series_path = find_series_path(temp_dir.path())?;
     return import_series(stack, matches, Some(series_path.as_path()));
 }
 
-#[cfg(feature = "import-compressed")]
 fn import_tar_series(stack: Stack, matches: &clap::ArgMatches, source_path: &Path) -> Result<()> {
     let source_file = std::fs::File::open(source_path)?;
     let mut archive = tar::Archive::new(source_file);
@@ -342,27 +339,6 @@ fn import_tar_series(stack: Stack, matches: &clap::ArgMatches, source_path: &Pat
     return import_series(stack, matches, Some(series_path.as_path()));
 }
 
-#[cfg(not(feature = "import-compressed"))]
-fn import_tgz_series(_: Stack, _: &clap::ArgMatches, _: &Path) -> Result<()> {
-    Err(anyhow!(
-        "StGit not built with support for compressed series"
-    ))
-}
-
-#[cfg(not(feature = "import-compressed"))]
-fn import_tbz2_series(_: Stack, _: &clap::ArgMatches, _: &Path) -> Result<()> {
-    Err(anyhow!(
-        "StGit not built with support for compressed series"
-    ))
-}
-
-#[cfg(not(feature = "import-compressed"))]
-fn import_tar_series(_: Stack, _: &clap::ArgMatches, _: &Path) -> Result<()> {
-    Err(anyhow!(
-        "StGit not built with support for compressed series"
-    ))
-}
-
 fn import_series(
     stack: Stack,
     matches: &clap::ArgMatches,
@@ -433,7 +409,6 @@ fn import_series(
     Ok(())
 }
 
-#[cfg(feature = "import-compressed")]
 fn find_series_path(base: &Path) -> Result<PathBuf> {
     for entry in base.read_dir()? {
         let entry = entry?;
@@ -479,32 +454,16 @@ fn import_mail(stack: Stack, matches: &clap::ArgMatches, source_path: Option<&Pa
     Ok(())
 }
 
-#[cfg(feature = "import-compressed")]
 fn read_gz(source_file: std::fs::File, content: &mut Vec<u8>) -> Result<()> {
     flate2::read::GzDecoder::new(source_file).read_to_end(content)?;
     Ok(())
 }
 
-#[cfg(not(feature = "import-compressed"))]
-fn read_gz(_source_file: std::fs::File, _content: &mut Vec<u8>) -> Result<()> {
-    Err(anyhow!(
-        "StGit not built with support for compressed patches"
-    ))
-}
-
-#[cfg(feature = "import-compressed")]
 fn read_bz2(source_file: std::fs::File, content: &mut Vec<u8>) -> Result<()> {
-    bzip2::read::BzDecoder::new(source_file).read_to_end(content)?;
+    bzip2_rs::DecoderReader::new(source_file).read_to_end(content)?;
     Ok(())
 }
 
-#[cfg(not(feature = "import-compressed"))]
-fn read_bz2(_source_file: std::fs::File, _content: &mut Vec<u8>) -> Result<()> {
-    Err(anyhow!(
-        "StGit not built with support for compressed patches"
-    ))
-}
-
 fn import_file<'repo>(
     stack: Stack<'repo>,
     matches: &clap::ArgMatches,
@@ -703,12 +662,10 @@ fn create_patch<'repo>(
 }
 
 fn stripname(name: &str) -> &str {
-    name.trim_start_matches(|c: char| c.is_ascii_digit() || c == '-')
-        .rsplit_once(".diff")
-        .map_or_else(
-            || name.rsplit_once(".patch").map_or(name, |(name, _)| name),
-            |(name, _)| name,
-        )
+    let name = name.trim_start_matches(|c: char| c.is_ascii_digit() || c == '-');
+    name.strip_suffix(".diff")
+        .or_else(|| name.strip_suffix(".patch"))
+        .unwrap_or(name)
 }
 
 fn split_patch(content: Vec<u8>) -> Result<(BString, BString)> {
@@ -738,7 +695,7 @@ fn split_patch(content: Vec<u8>) -> Result<(BString, BString)> {
 mod test {
     use bstr::B;
 
-    use super::split_patch;
+    use super::{split_patch, stripname};
 
     #[test]
     fn patch_without_message() {
@@ -818,6 +775,30 @@ mod test {
         assert_eq!(message, "");
         assert!(diff.starts_with(b"--- filename.txt"));
     }
+
+    #[test]
+    fn patch_name_with_numbered_order_and_patch_extension() {
+        let name = String::from("01-patch-name.patch");
+        assert_eq!(stripname(&name), "patch-name");
+    }
+
+    #[test]
+    fn patch_name_with_numbered_order_and_diff_extension() {
+        let name = String::from("01-patch-name.diff");
+        assert_eq!(stripname(&name), "patch-name");
+    }
+
+    #[test]
+    fn patch_name_with_numbered_order_and_diff_patch_extension() {
+        let name = String::from("01-patch-name.diff.patch");
+        assert_eq!(stripname(&name), "patch-name.diff");
+    }
+
+    #[test]
+    fn patch_name_with_numbered_order_and_patch_diff_extension() {
+        let name = String::from("01-patch-name.patch.diff");
+        assert_eq!(stripname(&name), "patch-name.patch");
+    }
 }
 
 #[derive(Default, Debug)]
