@@ -224,11 +224,19 @@ impl Statuses {
 
     /// Check whether there are any files with unmerged conflicts.
     pub(crate) fn check_conflicts(&self) -> Result<()> {
+        self.check_conflicts_filter(|_| true)
+    }
+
+    /// Check for unmerged conflicts on files filtered by a predicate.
+    pub(crate) fn check_conflicts_filter<F>(&self, predicate: F) -> Result<()>
+    where
+        F: Fn(&StatusEntry) -> bool,
+    {
         if self
             .iter()
-            .any(|entry| matches!(entry.kind(), StatusEntryKind::Unmerged))
+            .any(|entry| matches!(entry.kind(), StatusEntryKind::Unmerged) && predicate(&entry))
         {
-            Err(crate::stack::Error::OutstandingConflicts.into())
+            Err(anyhow!("resolve outstanding conflicts first"))
         } else {
             Ok(())
         }
@@ -251,16 +259,16 @@ impl Statuses {
 
                 if index_dirty && worktree_dirty {
                     return Err(anyhow!(
-                        "Index and worktree not clean. Use `refresh` or `reset --hard`"
+                        "index and worktree not clean; use `refresh` or `reset --hard`"
                     ));
                 }
             }
 
             if index_dirty {
-                Err(anyhow!("Index not clean. Use `refresh` or `reset --hard`"))
+                Err(anyhow!("index not clean; use `refresh` or `reset --hard`"))
             } else if worktree_dirty {
                 Err(anyhow!(
-                    "Worktree not clean. Use `refresh` or `reset --hard`"
+                    "worktree not clean; use `refresh` or `reset --hard`"
                 ))
             } else {
                 panic!("expected either/both worktree or index to be dirty")
@@ -274,7 +282,7 @@ impl Statuses {
     pub(crate) fn check_index_clean(&self) -> Result<()> {
         for entry in self.iter() {
             if !matches!(entry.index_status(), Status::Unmodified) {
-                return Err(anyhow!("Index not clean. Use `refresh` or `reset --hard`"));
+                return Err(anyhow!("index not clean; use `refresh` or `reset --hard`"));
             }
         }
         Ok(())
@@ -288,7 +296,7 @@ impl Statuses {
         for entry in self.iter() {
             if !matches!(entry.worktree_status(), Status::Unmodified) {
                 return Err(anyhow!(
-                    "Worktree not clean. Use `refresh` or `reset --hard`"
+                    "worktree not clean; use `refresh` or `reset --hard`"
                 ));
             }
         }
@@ -395,7 +403,7 @@ impl<'s> StatusHeaders<'s> {
     ///
     /// Panics if branch headers were not selected in the status options.
     #[allow(unused)]
-    pub(crate) fn branch_oid(&self) -> Option<git2::Oid> {
+    pub(crate) fn branch_oid(&self) -> Option<gix::ObjectId> {
         for entry in self.iter() {
             match entry.kind_value() {
                 (HeaderKind::BranchOid, b"(initial)") => return None,
@@ -665,7 +673,7 @@ mod tests {
         let headers = statuses.headers();
         assert_eq!(
             headers.branch_oid(),
-            Some(git2::Oid::from_str("ac9664e17984145e6fc238b8193686a1eef0feb2").unwrap())
+            Some(gix::ObjectId::from_hex(b"ac9664e17984145e6fc238b8193686a1eef0feb2").unwrap())
         );
         assert_eq!(headers.branch_head(), Some("main".to_string()));
         assert_eq!(headers.branch_upstream(), Some("origin/master".to_string()));
